package com.example.supermanoul.swimncoach.Fragments;


import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.Snackbar;
import android.support.design.widget.TextInputEditText;
import android.support.v4.app.Fragment;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.Toast;

import com.example.supermanoul.swimncoach.POJO.Group;
import com.example.supermanoul.swimncoach.POJO.Student;
import com.example.supermanoul.swimncoach.R;
import com.example.supermanoul.swimncoach.activities.Aplicacion;
import com.example.supermanoul.swimncoach.activities.MainActivity;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DatabaseReference;

import static com.facebook.FacebookSdk.getApplicationContext;

/**
 * A simple {@link Fragment} subclass.
 */
public class AddStudentFragment extends Fragment {

    private TextInputEditText studentName, studentEmail;
    private Button addStudentButton;
    private View addStudentFragmentLayout;
    private Bundle bundle;
    private String groupName = "";
    private FirebaseAuth auth2 = ((Aplicacion) getApplicationContext()).getAuth2();

    // Add TextWatcher to check input from user
    private TextWatcher tw = new TextWatcher() {

        @Override
        public void beforeTextChanged(CharSequence charSequence, int i, int i2, int i3) {
        }

        @Override
        public void onTextChanged(CharSequence charSequence, int i, int i2, int i3) {
        }

        @Override
        public void afterTextChanged(Editable editable) {
            // check Fields For Empty Values
            checkFieldsForEmptyValues();
        }
    };


    public AddStudentFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        //Set title bar
        ((MainActivity) getActivity()).setActionBarTitle(getActivity().getString(R.string.fragment_title_addStudent));

        // Inflate the layout for this fragment
        addStudentFragmentLayout = inflater.inflate(R.layout.fragment_add_student, container, false);

        //Get objects reference from layout
        studentName = (TextInputEditText) addStudentFragmentLayout.findViewById(R.id.studentName);
        studentEmail = (TextInputEditText) addStudentFragmentLayout.findViewById(R.id.studentEmail);
        addStudentButton = (Button) addStudentFragmentLayout.findViewById(R.id.addStudentButton);

        // Set listeners
        studentName.addTextChangedListener(tw);
        studentEmail.addTextChangedListener(tw);
        addStudentButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                addStudentButtonClicked(v);
            }
        });

        // Disable button if empty
        checkFieldsForEmptyValues();

        //Get groupName and set it
        bundle = this.getArguments();
        if (bundle != null) {
            groupName = bundle.getString("groupName");
            System.out.println("LLEGO HASTA EL BUNDLE");
            System.out.println(groupName);
        }

        return addStudentFragmentLayout;
    }


    private void checkFieldsForEmptyValues() {
        Button b = (Button) addStudentFragmentLayout.findViewById(R.id.addStudentButton);

        String s1 = studentName.getText().toString();
        String s2 = studentEmail.getText().toString();

        if (s1.equals("") || s2.equals("") || !isValidEmail(s2)) {
            b.setEnabled(false);
            b.setAlpha(.5f);
        } else {
            b.setEnabled(true);
            b.setAlpha(1f);
        }
    }

    // Check valid e-mail
    public final static boolean isValidEmail(CharSequence target) {
        return !TextUtils.isEmpty(target) && android.util.Patterns.EMAIL_ADDRESS.matcher(target).matches();
    }

    // "Add" Button clicked
    private void addStudentButtonClicked(final View v) {

        final String newStudentEmail = studentEmail.getText().toString();
        final String newStudentName = studentName.getText().toString();

        //Hide keyboard after button clicked
        final InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(getView().getWindowToken(), 0);


        auth2.createUserWithEmailAndPassword(newStudentEmail,"defaultPassword123").addOnCompleteListener(getActivity(), new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {

                        if (!task.isSuccessful()) {
                            String exception = task.getException().toString();
                            Toast.makeText(getActivity(), R.string.email_fail, Toast.LENGTH_SHORT).show();
                        }
                        else
                        {
                            String newStudentUID = auth2.getCurrentUser().getUid();
                            Student newStudent = new Student(newStudentUID, newStudentName, newStudentEmail);

                            auth2.sendPasswordResetEmail(newStudentEmail);
                            auth2.signOut();
                            saveStudentToDatabase(newStudent);
                            Snackbar snackbar = Snackbar.make(v, R.string.student_added, Snackbar.LENGTH_LONG);
                            snackbar.show();

                            //Go back to the latest fragment
                            getFragmentManager().popBackStackImmediate();

                        }

                    }
                });

    }

    void saveStudentToDatabase(Student newStudent) {

        // Create group ALL
        DatabaseReference groupsReference = ((Aplicacion) getApplicationContext()).getGroupsReference().child("ALL");
        groupsReference.setValue(new Group("ALL"));

        // Add student to ALL group
        DatabaseReference studentReference = ((Aplicacion) getApplicationContext()).getStudentsReference().child("ALL").child(newStudent.getId());
        studentReference.setValue(newStudent);

        // Add student to studentStats
        DatabaseReference studentStatsReference = ((Aplicacion) getApplicationContext()).getStudentsStatsReference().child(newStudent.getId());
        studentStatsReference.setValue(newStudent);

        // Create group withoutGroup
        groupsReference = ((Aplicacion) getApplicationContext()).getGroupsReference().child("NO_GROUP");
        groupsReference.setValue(new Group("NO_GROUP"));

        // Add student to NO_GROUP
        studentReference = ((Aplicacion) getApplicationContext()).getStudentsReference().child("NO_GROUP").child(newStudent.getId());
        studentReference.setValue(newStudent);

    }


    // Replaces '.' with ',' to store in the database
    public static String EncodeString(String string) {
        return string.replace(".", ",");
    }

    // Replace ',' with '.'
    public static String DecodeString(String string) {
        return string.replace(",", ".");
    }

    private void createStudentAccount(final String email, String password)
    {

    }


}
