package com.example.supermanoul.swimncoach.Fragments;


import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.Snackbar;
import android.support.design.widget.TextInputEditText;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.Toast;

import com.example.supermanoul.swimncoach.POJO.Group;
import com.example.supermanoul.swimncoach.POJO.Student;
import com.example.supermanoul.swimncoach.R;
import com.example.supermanoul.swimncoach.activities.Aplicacion;
import com.example.supermanoul.swimncoach.activities.MainActivity;
import com.example.supermanoul.swimncoach.adapters.StudentsToSelectAdapter;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DatabaseReference;

import static com.facebook.FacebookSdk.getApplicationContext;

/**
 * A simple {@link Fragment} subclass.
 */
public class AddStudentToGroupFragment extends Fragment {

    private TextInputEditText studentName, studentEmail;
    private Button addStudentButton;
    private View addStudentToGroupFragmentLayout;
    private Bundle bundle;
    private String groupName = "";
    private FirebaseAuth auth2 = ((Aplicacion) getApplicationContext()).getAuth2();
    private RecyclerView recyclerViewStudents;
    private RecyclerView.Adapter adapterStudents;
    private RecyclerView.LayoutManager layoutManagerStudents;
    private DatabaseReference studentsReference;

    // Add TextWatcher to check input from user
    private TextWatcher tw = new TextWatcher(){

        @Override
        public void beforeTextChanged(CharSequence charSequence, int i, int i2, int i3) {
        }

        @Override
        public void onTextChanged(CharSequence charSequence, int i, int i2, int i3) {
        }

        @Override
        public void afterTextChanged(Editable editable) {
            // check Fields For Empty Values
            checkFieldsForEmptyValues();
        }
    };


    public AddStudentToGroupFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        //Set title bar
        ((MainActivity) getActivity()).setActionBarTitle(getActivity().getString(R.string.fragment_title_addStudentToGroup));

        // Inflate the layout for this fragment
        addStudentToGroupFragmentLayout = inflater.inflate(R.layout.fragment_add_student_to_group, container, false);

        recyclerViewStudents = (RecyclerView) addStudentToGroupFragmentLayout.findViewById(R.id.recycler_view_studentsWithNoGroup);

        // use this setting to improve performance if you know that changes
        // in content do not change the layout size of the RecyclerView
        recyclerViewStudents.setHasFixedSize(true);

        // use a linear layout manager
        layoutManagerStudents = new LinearLayoutManager(getActivity());
        recyclerViewStudents.setLayoutManager(layoutManagerStudents);

        //Database reference
        studentsReference = ((Aplicacion) getApplicationContext()).getStudentsReference().child("NO_GROUP");

        //Get groupName and set it
        bundle = this.getArguments();
        if(bundle != null){
            groupName = bundle.getString("groupName");
        }

        // specify an adapter (see also next example)
        adapterStudents = new StudentsToSelectAdapter(getActivity(), studentsReference, groupName, getFragmentManager());
        recyclerViewStudents.setAdapter(adapterStudents);

        //Get objects reference from layout
        studentName = (TextInputEditText) addStudentToGroupFragmentLayout.findViewById(R.id.studentName);
        studentEmail = (TextInputEditText) addStudentToGroupFragmentLayout.findViewById(R.id.studentEmail);
        addStudentButton = (Button) addStudentToGroupFragmentLayout.findViewById(R.id.addStudentButton);

        // Set listeners
        studentName.addTextChangedListener(tw);
        studentEmail.addTextChangedListener(tw);
        addStudentButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v){
                addStudentButtonClicked(v);
            }
        });

        // Disable button if empty
        checkFieldsForEmptyValues();

        return addStudentToGroupFragmentLayout;
    }


    private void checkFieldsForEmptyValues(){
        Button b = (Button) addStudentToGroupFragmentLayout.findViewById(R.id.addStudentButton);

        String s1 = studentName.getText().toString();
        String s2 = studentEmail.getText().toString();

        if(s1.equals("") || s2.equals("") || !isValidEmail(s2)){
            b.setEnabled(false);
            b.setAlpha(.5f);

        } else {
            b.setEnabled(true);
            b.setAlpha(1f);
        }
    }

    // Check valid e-mail
    public final static boolean isValidEmail(CharSequence target) {
        return !TextUtils.isEmpty(target) && android.util.Patterns.EMAIL_ADDRESS.matcher(target).matches();
    }

    // "Add" Button clicked
    private void addStudentButtonClicked( View v){

        //Hide keyboard after button clicked
        final InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(getView().getWindowToken(), 0);

        final String newStudentEmail = studentEmail.getText().toString();
        final String newStudentName = studentName.getText().toString();

        auth2.createUserWithEmailAndPassword(newStudentEmail,"defaultPassword123").addOnCompleteListener(getActivity(), new OnCompleteListener<AuthResult>() {
            @Override
            public void onComplete(@NonNull Task<AuthResult> task) {

                if (!task.isSuccessful()) {
                    String exception = task.getException().toString();
                    Toast.makeText(getActivity(), R.string.email_fail, Toast.LENGTH_SHORT).show();
                }
                else
                {
                    String newStudentUID = auth2.getCurrentUser().getUid();
                    Student newStudent = new Student(newStudentUID, newStudentName, newStudentEmail);

                    auth2.sendPasswordResetEmail(newStudentEmail);
                    auth2.signOut();
                    saveStudentToDatabase(newStudent);
                    Snackbar snackbar = Snackbar.make(getView(), R.string.student_added, Snackbar.LENGTH_LONG);
                    snackbar.show();

                    //Go back to the latest fragment
                    getFragmentManager().popBackStackImmediate();

                }

            }
        });


    }

    void saveStudentToDatabase(Student newStudent) {

        // Create group ALL
        DatabaseReference groupsReference=((Aplicacion) getApplicationContext()).getGroupsReference().child("ALL");
        groupsReference.setValue(new Group("ALL"));

        // Add student to ALL group
        DatabaseReference studentReference=((Aplicacion) getApplicationContext()).getStudentsReference().child("ALL").child(newStudent.getId());
        studentReference.setValue(newStudent);

        // Add student to studentStats
        DatabaseReference studentStatsReference = ((Aplicacion) getApplicationContext()).getStudentsStatsReference().child(newStudent.getId());
        studentStatsReference.setValue(newStudent);

        // Add student to corresponding group
        if(!groupName.equals("") && groupName != null){
            studentReference=((Aplicacion) getApplicationContext()).getStudentsReference().child(groupName).child(newStudent.getId());
            studentReference.setValue(newStudent);

            studentStatsReference = studentStatsReference.child("groups").child(groupName);
            studentStatsReference.setValue(new Group(groupName));
        }
        else{
            // Create group withoutGroup
            groupsReference=((Aplicacion) getApplicationContext()).getGroupsReference().child("NO_GROUP");
            groupsReference.setValue(new Group("NO_GROUP"));

            // Add student to ALL group
            studentReference=((Aplicacion) getApplicationContext()).getStudentsReference().child("NO_GROUP").child(newStudent.getId());
            studentReference.setValue(newStudent);
        }



    }

    // Replaces '.' with ',' to store in the database
    public static String EncodeString(String string) {
        return string.replace(".", ",");
    }


}
