package com.example.supermanoul.swimncoach.Fragments;


import android.app.DatePickerDialog;
import android.app.TimePickerDialog;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TimePicker;

import com.example.supermanoul.swimncoach.POJO.Group;
import com.example.supermanoul.swimncoach.POJO.TrainingSession;
import com.example.supermanoul.swimncoach.R;
import com.example.supermanoul.swimncoach.activities.Aplicacion;
import com.example.supermanoul.swimncoach.activities.MainActivity;
import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Locale;

import static com.facebook.FacebookSdk.getApplicationContext;

/**
 * A simple {@link Fragment} subclass.
 */
public class AddTrainingSessionFragment extends Fragment implements ChildEventListener, AdapterView.OnItemSelectedListener, View.OnClickListener {

    private ArrayList<String> keys = new ArrayList<String>();
    private ArrayList<DataSnapshot> items = new ArrayList<DataSnapshot>();
    private DatabaseReference groupsReference;
    private Spinner groupSpinner;
    private DatePickerDialog dateDialog;
    private TimePickerDialog timeDialog;
    private SimpleDateFormat dateFormatter;
    private SimpleDateFormat timeFormatter;
    private EditText dateEditText;
    private EditText timeEditText;
    private EditText durationEditText;
    private EditText poolLengthEditText;
    private Button deleteSessionButton;
    private Button saveButton;
    private String groupSelected = "";
    private ArrayAdapter<String> adapter;
    private View addTSFragmentLayout;


    // Add TextWatcher to check input from user
    private TextWatcher tw = new TextWatcher() {

        @Override
        public void beforeTextChanged(CharSequence charSequence, int i, int i2, int i3) {
        }

        @Override
        public void onTextChanged(CharSequence charSequence, int i, int i2, int i3) {
        }

        @Override
        public void afterTextChanged(Editable editable) {
            // check Fields For Empty Values
            checkFieldsForEmptyValues();
        }
    };


    public AddTrainingSessionFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        addTSFragmentLayout = inflater.inflate(R.layout.fragment_add_training_session, container, false);

        // Set title bar
        ((MainActivity) getActivity()).setActionBarTitle(getActivity().getString(R.string.fragment_title_addTrainingSession));

        // Add reference to layout objects
        groupSpinner = (Spinner) addTSFragmentLayout.findViewById(R.id.spinnerGroupTS);
        dateEditText = (EditText) addTSFragmentLayout.findViewById(R.id.editTextDateTS);
        timeEditText = (EditText) addTSFragmentLayout.findViewById(R.id.editTextTimeTS);
        poolLengthEditText = (EditText) addTSFragmentLayout.findViewById(R.id.editTextPoolLengthTS);
        durationEditText = (EditText) addTSFragmentLayout.findViewById(R.id.editTextDurationTS);
        deleteSessionButton = (Button) addTSFragmentLayout.findViewById(R.id.deleteSessionButton);
        saveButton = (Button) addTSFragmentLayout.findViewById(R.id.saveSessionButton);

        // If coming here from Main fragment using calendar and already have a selected date
        Bundle b = this.getArguments();
        if(b != null){
            dateEditText.setText(b.getString("date"));
        }


        // Add reference to database and listener
        groupsReference = ((Aplicacion) getApplicationContext()).getGroupsReference();
        groupsReference.addChildEventListener(this);

        // Adapter for groupSpinner
        adapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_item, keys);
        groupSpinner.setAdapter(adapter);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);


        //Listeners
        groupSpinner.setOnItemSelectedListener(this);
        dateEditText.setOnClickListener(this);
        dateEditText.addTextChangedListener(tw);
        timeEditText.setOnClickListener(this);
        timeEditText.addTextChangedListener(tw);
        poolLengthEditText.setOnClickListener(this);
        poolLengthEditText.addTextChangedListener(tw);
        durationEditText.setOnClickListener(this);
        durationEditText.addTextChangedListener(tw);
        saveButton.setOnClickListener(this);
        deleteSessionButton.setOnClickListener(this);

        //Create and set dialogs for date and time
        dateFormatter = new SimpleDateFormat("dd-MM-yyyy", Locale.US);
        timeFormatter = new SimpleDateFormat("HH:mm", Locale.US);
        setDateTimeField();

        // Disable button if empty
        checkFieldsForEmptyValues();

        return addTSFragmentLayout;
    }

    @Override
    public void onChildAdded(DataSnapshot dataSnapshot, String s) {
        if (!dataSnapshot.getKey().equals("ALL") && !dataSnapshot.getKey().equals("NO_GROUP")) {
            items.add(dataSnapshot);
            keys.add(dataSnapshot.getKey());
            adapter.notifyDataSetChanged();
        }

    }

    @Override
    public void onChildChanged(DataSnapshot dataSnapshot, String s) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            items.set(index, dataSnapshot);
        }
    }

    @Override
    public void onChildRemoved(DataSnapshot dataSnapshot) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            keys.remove(index);
            items.remove(index);
        }
    }

    @Override
    public void onChildMoved(DataSnapshot dataSnapshot, String s) {
    }

    @Override
    public void onCancelled(DatabaseError databaseError) {
    }

    public DatabaseReference getRef(int pos) {
        return items.get(pos).getRef();
    }

    public Group getItem(int pos) {
        return items.get(pos).getValue(Group.class);
    }


    // On Group selected from spinner
    public void onItemSelected(AdapterView<?> parent, View view,
                               int pos, long id) {
        // An item was selected. You can retrieve the selected item using
        // parent.getItemAtPosition(pos)
        groupSelected = parent.getItemAtPosition(pos).toString();
    }

    public void onNothingSelected(AdapterView<?> parent) {
        // Another interface callback
    }

    // Set date and time dialogs
    private void setDateTimeField() {

        Calendar newCalendar = Calendar.getInstance();

        dateDialog = new DatePickerDialog(getActivity(), new DatePickerDialog.OnDateSetListener() {

            public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
                Calendar newDate = Calendar.getInstance();
                newDate.set(year, monthOfYear, dayOfMonth);
                dateEditText.setText(dateFormatter.format(newDate.getTime()));
            }

        }, newCalendar.get(Calendar.YEAR), newCalendar.get(Calendar.MONTH), newCalendar.get(Calendar.DAY_OF_MONTH));

        timeDialog = new TimePickerDialog(getActivity(), new TimePickerDialog.OnTimeSetListener() {
            @Override
            public void onTimeSet(TimePicker timePicker, int selectedHour, int selectedMinute) {
                Calendar newTime = Calendar.getInstance();
                newTime.set(0,0,0,selectedHour,selectedMinute);
                timeEditText.setText(timeFormatter.format(newTime.getTime()));

            }
        }, newCalendar.get(Calendar.HOUR_OF_DAY), newCalendar.get(Calendar.MINUTE), true);//Yes 24 hour time
    }

    @Override
    public void onClick(View view) {
        if (view == dateEditText) {
            dateDialog.show();
        }
        else if (view == timeEditText){
            timeDialog.show();
        }
        else if (view == deleteSessionButton){

            //Go back to the latest fragment
            getFragmentManager().popBackStackImmediate();
        }
        else if (view == saveButton){

            saveTrainingSession();
            //Go back to the latest fragment
            getFragmentManager().popBackStackImmediate();
        }
    }

    private void checkFieldsForEmptyValues() {
        Button b = (Button) addTSFragmentLayout.findViewById(R.id.saveSessionButton);

        String s1 = dateEditText.getText().toString();
        String s2 = timeEditText.getText().toString();
        String s3 = poolLengthEditText.getText().toString();
        String s4 = durationEditText.getText().toString();
        String s5 = groupSelected;


        if (s1.length() != 10 ||
                s2.length() != 5 ||
                s3.length() > 3 || s3.length() <1 ||
                s4.length() > 3 || s4.length() < 1 ||
                s5 == null || s5.length() == 0 ) {
            b.setEnabled(false);
            b.setAlpha(.5f);
        } else {
            b.setEnabled(true);
            b.setAlpha(1);
        }
    }

    private void saveTrainingSession(){
        String s1 = dateEditText.getText().toString();
        String s2 = timeEditText.getText().toString();
        String s3 = poolLengthEditText.getText().toString();
        String s4 = durationEditText.getText().toString();
        String s5 = groupSelected;

        String TSid = s1+"_"+s2;
        DatabaseReference TSreference = ((Aplicacion) getApplicationContext()).getTrainingSessionsReference().child(groupSelected).child(TSid);
        DatabaseReference TSreference2 = ((Aplicacion) getApplicationContext()).getTrainingSessionsReference().child("ALL").child(TSid);
        TSreference.setValue(new TrainingSession(TSid,s1,s2,s5,Integer.parseInt(s3),Integer.parseInt(s4),false));
        TSreference2.setValue(new TrainingSession(TSid,s1,s2,s5,Integer.parseInt(s3),Integer.parseInt(s4),false));
    }


}


