package com.example.supermanoul.swimncoach.Fragments;


import android.content.DialogInterface;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.EditText;

import com.example.supermanoul.swimncoach.POJO.Group;
import com.example.supermanoul.swimncoach.R;
import com.example.supermanoul.swimncoach.activities.Aplicacion;
import com.example.supermanoul.swimncoach.activities.MainActivity;
import com.example.supermanoul.swimncoach.adapters.GroupsAdapter;
import com.google.firebase.database.DatabaseReference;

import static com.facebook.FacebookSdk.getApplicationContext;


/**
 * A simple {@link Fragment} subclass.
 */
public class GroupsFragment extends Fragment implements OnClickListener {

    private RecyclerView recyclerViewGroups;
    private RecyclerView.Adapter adapterGroups;
    private RecyclerView.LayoutManager layoutManagerGroups;

    private DatabaseReference groupsReference = ((Aplicacion) getApplicationContext()).getGroupsReference();

    public GroupsFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View groupsFragmentLayout = inflater.inflate(R.layout.fragment_groups, container, false);

        recyclerViewGroups = (RecyclerView) groupsFragmentLayout.findViewById(R.id.recycler_view_groups);

        // use this setting to improve performance if you know that changes
        // in content do not change the layout size of the RecyclerView
        recyclerViewGroups.setHasFixedSize(true);

        // use a linear layout manager
        layoutManagerGroups = new LinearLayoutManager(getActivity());
        recyclerViewGroups.setLayoutManager(layoutManagerGroups);

        // specify an adapter (see also next example)
        adapterGroups = new GroupsAdapter(getActivity(), groupsReference);
        recyclerViewGroups.setAdapter(adapterGroups);

        // Listener for fab
        FloatingActionButton fabGroups = (FloatingActionButton) groupsFragmentLayout.findViewById(R.id.fab_groups);
        fabGroups.bringToFront();
        fabGroups.setOnClickListener(this);

        // Set title bar
        ((MainActivity) getActivity()).setActionBarTitle(getActivity().getString(R.string.fragment_title_groups));

        return groupsFragmentLayout;

    }

    public void onClick(final View v) {

        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        final EditText edittext = new EditText(getActivity());
        edittext.setMaxLines(1);
        edittext.setInputType(InputType.TYPE_CLASS_TEXT);
        builder.setMessage(R.string.ad_textNewGroup);
        builder.setTitle(R.string.ad_newGroup);

        builder.setView(edittext);

        builder.setPositiveButton(R.string.save, new DialogInterface.OnClickListener() {

            public void onClick(DialogInterface dialog, int whichButton) {
                String EditTextValue = edittext.getText().toString();

                //Send group's name to next fragment
                Bundle b = new Bundle();
                b.putString("groupName", EditTextValue);

                AddGroupFragment fragment = new AddGroupFragment();
                fragment.setArguments(b);

                // Save group to database
                DatabaseReference groupsReference=((Aplicacion) getApplicationContext()).getGroupsReference().child(EditTextValue);
                groupsReference.setValue(new Group(EditTextValue));

                // Notify of a group added
                Snackbar snackbar = Snackbar.make(v, R.string.group_created, Snackbar.LENGTH_LONG);
                snackbar.show();

                FragmentTransaction ft = getFragmentManager().beginTransaction();
                ft.replace(R.id.fragment_container, fragment);
                ft.addToBackStack(null);
                ft.commit();

            }
        });

        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
            }
        });

        final AlertDialog alert = builder.show();
        alert.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);

        // Add TextWatcher to check input from user
        TextWatcher tw = new TextWatcher(){

            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i2, int i3) {
            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i2, int i3) {
            }

            @Override
            public void afterTextChanged(Editable editable) {
                // check Fields For Empty Values
                if(!edittext.getText().toString().equals("")){
                    alert.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(true);
                }
                else{
                    alert.getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
                }
            }

        };

        // Set listener
        edittext.addTextChangedListener(tw);



    }



}
