package com.example.supermanoul.swimncoach.adapters;

import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.example.supermanoul.swimncoach.Fragments.AddGroupFragment;
import com.example.supermanoul.swimncoach.POJO.Group;
import com.example.supermanoul.swimncoach.R;
import com.example.supermanoul.swimncoach.activities.Aplicacion;
import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static com.facebook.FacebookSdk.getApplicationContext;



public class GroupsAdapter extends RecyclerView.Adapter<GroupsAdapter.ViewHolder> implements ChildEventListener{
    private List<Group> groups = Collections.emptyList();
    private LayoutInflater inflater;
    private ArrayList<String> keys = new ArrayList<String>();
    private ArrayList<DataSnapshot> items = new ArrayList<DataSnapshot>();
    private DatabaseReference groupsReference;
    private DatabaseReference getStudentsInGroupReference;



    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder
    public static class ViewHolder extends RecyclerView.ViewHolder {
        TextView groupName;
        TextView groupNumberStudents;

        public ViewHolder(View itemView) {

            super(itemView);
            groupName = (TextView) itemView.findViewById(R.id.groupListName);
            groupNumberStudents = (TextView) itemView.findViewById(R.id.groupListNumberStudents);

        }
    }

    // Provide a suitable constructor (depends on the kind of dataset)
    public GroupsAdapter(Context context, DatabaseReference reference) {

        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();
        groupsReference = reference;
        inflater = LayoutInflater.from(context);
        groupsReference.addChildEventListener(this);
    }

    // Create new views (invoked by the layout manager)
    @Override
    public GroupsAdapter.ViewHolder onCreateViewHolder(ViewGroup parent,
                                                   int viewType) {
        View view = inflater.inflate(R.layout.group_row, parent, false);
        ViewHolder holder = new ViewHolder(view);
        return holder;
    }

    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(final ViewHolder holder, int position) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element
        final Group group = getItem(position);
        holder.groupName.setText(group.getGroupName());

        // Reference to get the number of students in group
        getStudentsInGroupReference = ((Aplicacion) getApplicationContext()).getStudentsReference().child(group.getGroupName());

        // Listener to get number of students in group
        getStudentsInGroupReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                holder.groupNumberStudents.setText(String.valueOf(dataSnapshot.getChildrenCount()) + " "+ getApplicationContext().getString(R.string.students));
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });


        holder.itemView.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View view){
                AppCompatActivity activity = (AppCompatActivity) view.getContext();

                //Send group's name to next fragment
                Bundle b = new Bundle();
                b.putString("groupName", group.getGroupName());

                AddGroupFragment fragment = new AddGroupFragment();
                fragment.setArguments(b);

                FragmentTransaction ft = activity.getSupportFragmentManager().beginTransaction();
                ft.replace(R.id.fragment_container, fragment);
                ft.addToBackStack(null);
                ft.commit();
            }
        });
    }

    @Override
    public void onChildAdded(DataSnapshot dataSnapshot, String s) {
        if(!dataSnapshot.getKey().equals("ALL") && !dataSnapshot.getKey().equals("NO_GROUP")){
            items.add(dataSnapshot);
            keys.add(dataSnapshot.getKey());
            notifyItemInserted(getItemCount()-1);
        }

    }

    @Override
    public void onChildChanged(DataSnapshot dataSnapshot, String s) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            items.set(index, dataSnapshot);
            notifyItemChanged(index, dataSnapshot.getValue(Group.class));
        }
    }

    @Override
    public void onChildRemoved(DataSnapshot dataSnapshot) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            keys.remove(index);
            items.remove(index);
            notifyItemRemoved(index);
        }
    }

    @Override
    public void onChildMoved(DataSnapshot dataSnapshot, String s) {
    }

    @Override
    public void onCancelled(DatabaseError databaseError) {
    }

    @Override public int getItemCount() {
        return items.size();
    }

    public DatabaseReference getRef(int pos) {
        return items.get(pos).getRef();
    }

    public Group getItem(int pos) {
        return items.get(pos).getValue(Group.class);
    }
}
