package com.example.supermanoul.swimncoach.adapters;

import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.example.supermanoul.swimncoach.Fragments.ShowResultsFragment;
import com.example.supermanoul.swimncoach.Fragments.TrainingSessionSelectedFragment;
import com.example.supermanoul.swimncoach.POJO.TrainingSession;
import com.example.supermanoul.swimncoach.R;
import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.util.ArrayList;

import static com.example.supermanoul.swimncoach.R.color.lightGray;
import static com.facebook.FacebookSdk.getApplicationContext;



public class MainAdapter extends RecyclerView.Adapter<MainAdapter.ViewHolder> implements ChildEventListener{
    private LayoutInflater inflater;

    private ArrayList<String> keys = new ArrayList<String>();
    private ArrayList<DataSnapshot> items = new ArrayList<DataSnapshot>();
    private DatabaseReference TSReference;
    private String dateSelected;

    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder
    public static class ViewHolder extends RecyclerView.ViewHolder {
        TextView TSdate;
        TextView TStime;
        TextView TSgroup;
        TextView TSpoolLength;
        TextView TSduration;
        TextView TSdone;

        public ViewHolder(View itemView) {

            super(itemView);
            TSdate = (TextView) itemView.findViewById(R.id.TSListDate);
            TStime = (TextView) itemView.findViewById(R.id.TSListTime);
            TSgroup = (TextView) itemView.findViewById(R.id.TSListGroup);
            TSpoolLength = (TextView) itemView.findViewById(R.id.TSListPoolLength);
            TSduration = (TextView) itemView.findViewById(R.id.TSListDuration);
            TSdone = (TextView) itemView.findViewById(R.id.TSDone);


        }
    }

    // Provide a suitable constructor (depends on the kind of dataset)
    public MainAdapter(Context context, DatabaseReference reference, String date) {
        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();
        TSReference = reference;
        this.dateSelected = date;
        inflater = LayoutInflater.from(context);

        //Add event listener to childs
        TSReference.addChildEventListener(this);
    }

    // Create new views (invoked by the layout manager)
    @Override
    public MainAdapter.ViewHolder onCreateViewHolder(ViewGroup parent,
                                                                 int viewType) {
        View view = inflater.inflate(R.layout.training_session_row, parent, false);
        MainAdapter.ViewHolder holder = new MainAdapter.ViewHolder(view);
        return holder;
    }

    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(MainAdapter.ViewHolder holder, int position) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element
        final TrainingSession ts = getItem(position);
        holder.TSdate.setText(getApplicationContext().getString(R.string.date) + ": "+ ts.getDate());
        holder.TStime.setText(getApplicationContext().getString(R.string.time) + ": "+ ts.getTime());
        holder.TSgroup.setText(getApplicationContext().getString(R.string.group) + ": " + ts.getGroup());
        holder.TSpoolLength.setText(getApplicationContext().getString(R.string.pool_length) + ": " + ts.getPoolSize() + " " + getApplicationContext().getString(R.string.m));
        holder.TSduration.setText(getApplicationContext().getString(R.string.duration) + ": " + ts.getDuration() + " " + getApplicationContext().getString(R.string.min));

        if(ts.isFinished()){
            holder.TSdone.setVisibility(View.VISIBLE);
            //holder.itemView.setEnabled(false);

            holder.itemView.setBackgroundColor(getApplicationContext().getResources().getColor(lightGray));

            holder.TSdate.setEnabled(false);
            holder.TStime.setEnabled(false);
            holder.TSgroup.setEnabled(false);
            holder.TSpoolLength.setEnabled(false);
            holder.TSduration.setEnabled(false);

            holder.itemView.setOnClickListener(new View.OnClickListener(){
                @Override
                public void onClick(View view){
                    AppCompatActivity activity = (AppCompatActivity) view.getContext();

                    //Send tsID to next fragment
                    Bundle b = new Bundle();
                    b.putString("training_session_id", ts.getId());
                    b.putString("groupName", ts.getGroup());

                    ShowResultsFragment fragment = new ShowResultsFragment();
                    fragment.setArguments(b);

                    FragmentTransaction ft = activity.getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.fragment_container, fragment);
                    ft.addToBackStack(null);
                    ft.commit();
                }
            });
        }

        else{
            holder.itemView.setOnClickListener(new View.OnClickListener(){
                @Override
                public void onClick(View view){
                    AppCompatActivity activity = (AppCompatActivity) view.getContext();

                    //Send group's name to next fragment
                    Bundle b = new Bundle();
                    b.putString("id", ts.getId());
                    b.putString("date", ts.getDate());
                    b.putString("time", ts.getTime());
                    b.putString("groupName", ts.getGroup());
                    b.putInt("poolLength", ts.getPoolSize());
                    b.putInt("duration", ts.getDuration());

                    TrainingSessionSelectedFragment fragment = new TrainingSessionSelectedFragment();
                    fragment.setArguments(b);

                    FragmentTransaction ft = activity.getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.fragment_container, fragment);
                    ft.addToBackStack(null);
                    ft.commit();
                }
            });
        }

    }

    @Override
    public void onChildAdded(DataSnapshot dataSnapshot, String s) {
        if(dataSnapshot.getKey().startsWith(dateSelected)){
            items.add(dataSnapshot);
            keys.add(dataSnapshot.getKey());
            notifyItemInserted(getItemCount()-1);
        }

    }

    @Override
    public void onChildChanged(DataSnapshot dataSnapshot, String s) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            items.set(index, dataSnapshot);
            notifyItemChanged(index, dataSnapshot.getValue(TrainingSession.class));
        }
    }

    @Override
    public void onChildRemoved(DataSnapshot dataSnapshot) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            keys.remove(index);
            items.remove(index);
            notifyItemRemoved(index);
        }
    }

    @Override
    public void onChildMoved(DataSnapshot dataSnapshot, String s) {
    }

    @Override
    public void onCancelled(DatabaseError databaseError) {
    }

    @Override public int getItemCount() {
        return items.size();
    }

    public DatabaseReference getRef(int pos) {
        return items.get(pos).getRef();
    }

    public TrainingSession getItem(int pos) {
        return items.get(pos).getValue(TrainingSession.class);
    }

    // Replace ',' with '.'
    public static String DecodeString(String string) {
        return string.replace(",", ".");
    }



}
