package com.example.supermanoul.swimncoach.adapters;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.example.supermanoul.swimncoach.POJO.Group;
import com.example.supermanoul.swimncoach.POJO.Results;
import com.example.supermanoul.swimncoach.R;
import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static com.facebook.FacebookSdk.getApplicationContext;



public class ResultsAdapter extends RecyclerView.Adapter<ResultsAdapter.ViewHolder> implements ChildEventListener{
    private List<Group> results = Collections.emptyList();
    private LayoutInflater inflater;
    private ArrayList<String> keys = new ArrayList<String>();
    private ArrayList<DataSnapshot> items = new ArrayList<DataSnapshot>();
    private DatabaseReference resultsReference;
    private String tsID = "";
    private String groupNameString = "";



    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder
    public static class ViewHolder extends RecyclerView.ViewHolder {
        TextView totalDistance;
        TextView averagePace;
        TextView averageSpeed;
        TextView studentName;
        TextView totalTime;
        TextView groupName;
        TextView dateAndTime;

        public ViewHolder(View itemView) {

            super(itemView);
            totalDistance = (TextView) itemView.findViewById(R.id.resultsTotalDistance);
            averagePace = (TextView) itemView.findViewById(R.id.resultsAveragePace);
            averageSpeed = (TextView) itemView.findViewById(R.id.resultsAverageSpeed);
            studentName = (TextView) itemView.findViewById(R.id.resultsStudentName);
            totalTime = (TextView) itemView.findViewById(R.id.resultsTotalTime);
            groupName = (TextView) itemView.findViewById(R.id.groupNameResults);
            dateAndTime = (TextView) itemView.findViewById(R.id.dateTSResults);


        }
    }

    // Provide a suitable constructor (depends on the kind of dataset)
    public ResultsAdapter(Context context, DatabaseReference reference) {

        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();
        resultsReference = reference;
        inflater = LayoutInflater.from(context);
        resultsReference.addChildEventListener(this);


    }

    // Create new views (invoked by the layout manager)
    @Override
    public ResultsAdapter.ViewHolder onCreateViewHolder(ViewGroup parent,
                                                       int viewType) {
        View view = inflater.inflate(R.layout.result_row, parent, false);
        ViewHolder holder = new ViewHolder(view);
        return holder;
    }

    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(final ViewHolder holder, int position) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element
        SimpleDateFormat formatter = new SimpleDateFormat("mm:ss.S");

        String total_distance_string = getApplicationContext().getResources().getString(R.string.total_distance);
        String total_time_string = getApplicationContext().getResources().getString(R.string.total_time);
        String average_speed_string = getApplicationContext().getResources().getString(R.string.average_speed);
        String average_pace_string = getApplicationContext().getResources().getString(R.string.average_pace);

        Results results = getItem(position);


        holder.studentName.setText(results.getStudentName());
        holder.totalTime.setText(String.format(total_time_string, results.getTotalTimeInString()));

        holder.averagePace.setText(String.format(average_pace_string, results.getAveragePaceInMs()));
        holder.averageSpeed.setText(String.format(average_speed_string, results.getAverageSpeed()));
        holder.totalDistance.setText(String.format(total_distance_string, Integer.toString(results.getTotalDistance())));


    }

    @Override
    public void onChildAdded(DataSnapshot dataSnapshot, String s) {
        items.add(dataSnapshot);
        keys.add(dataSnapshot.getKey());
        notifyItemInserted(getItemCount()-1);


    }

    @Override
    public void onChildChanged(DataSnapshot dataSnapshot, String s) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            items.set(index, dataSnapshot);
            notifyItemChanged(index, dataSnapshot.getValue(Group.class));
        }
    }

    @Override
    public void onChildRemoved(DataSnapshot dataSnapshot) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            keys.remove(index);
            items.remove(index);
            notifyItemRemoved(index);
        }
    }

    @Override
    public void onChildMoved(DataSnapshot dataSnapshot, String s) {
    }

    @Override
    public void onCancelled(DatabaseError databaseError) {
    }

    @Override public int getItemCount() {
        return items.size();
    }

    public DatabaseReference getRef(int pos) {
        return items.get(pos).getRef();
    }

    public Results getItem(int pos) {
        return items.get(pos).getValue(Results.class);
    }

}
