package com.example.supermanoul.swimncoach.adapters;

import android.content.Context;
import android.support.v4.app.FragmentManager;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.example.supermanoul.swimncoach.POJO.Group;
import com.example.supermanoul.swimncoach.POJO.Student;
import com.example.supermanoul.swimncoach.R;
import com.example.supermanoul.swimncoach.activities.Aplicacion;
import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.util.ArrayList;

import static com.facebook.FacebookSdk.getApplicationContext;



public class StudentsToSelectAdapter extends RecyclerView.Adapter<StudentsToSelectAdapter.ViewHolder> implements ChildEventListener{
    private LayoutInflater inflater;

    private ArrayList<String> keys = new ArrayList<String>();
    private ArrayList<DataSnapshot> items = new ArrayList<DataSnapshot>();
    private DatabaseReference studentsReference;
    private String groupName;
    private FragmentManager fm;

    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder
    public static class ViewHolder extends RecyclerView.ViewHolder {
        TextView studentName;
        TextView studentEmail;

        public ViewHolder(View itemView) {

            super(itemView);
            studentName = (TextView) itemView.findViewById(R.id.studentListName);
            studentEmail = (TextView) itemView.findViewById(R.id.studentListEmail);

        }
    }

    // Provide a suitable constructor (depends on the kind of dataset)
    public StudentsToSelectAdapter(Context context, DatabaseReference reference, String groupName, FragmentManager fm) {
        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();
        studentsReference = reference;
        this.groupName = groupName;
        this.fm = fm;
        inflater = LayoutInflater.from(context);
        studentsReference.addChildEventListener(this);
    }

    // Create new views (invoked by the layout manager)
    @Override
    public StudentsToSelectAdapter.ViewHolder onCreateViewHolder(ViewGroup parent,
                                                         int viewType) {
        View view = inflater.inflate(R.layout.student_row, parent, false);
        StudentsToSelectAdapter.ViewHolder holder = new StudentsToSelectAdapter.ViewHolder(view);
        return holder;
    }

    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(StudentsToSelectAdapter.ViewHolder holder, int position) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element
        final Student student = getItem(position);
        holder.studentEmail.setText(student.getEmail());
        holder.studentName.setText(student.getName());

        holder.itemView.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View view){
                AppCompatActivity activity = (AppCompatActivity) view.getContext();

                //Remove from NO_GROUP
                studentsReference.child(EncodeString(student.getId())).removeValue();

                //Add to new group
                studentsReference = ((Aplicacion) getApplicationContext()).getStudentsReference().child(groupName).child(EncodeString(student.getId()));
                studentsReference.setValue(student);

                // Add student to studentStats
                DatabaseReference studentStatsReference = ((Aplicacion) getApplicationContext()).getStudentsStatsReference().child(student.getId());
                studentStatsReference = studentStatsReference.child("groups").child(groupName);
                studentStatsReference.setValue(new Group(groupName));

                //Change fragment
                fm.popBackStackImmediate();

            }
        });
    }

    @Override
    public void onChildAdded(DataSnapshot dataSnapshot, String s) {
        items.add(dataSnapshot);
        keys.add(dataSnapshot.getKey());
        notifyItemInserted(getItemCount()-1);
    }

    @Override
    public void onChildChanged(DataSnapshot dataSnapshot, String s) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            items.set(index, dataSnapshot);
            notifyItemChanged(index, dataSnapshot.getValue(Student.class));
        }
    }

    @Override
    public void onChildRemoved(DataSnapshot dataSnapshot) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            keys.remove(index);
            items.remove(index);
            notifyItemRemoved(index);
        }
    }

    @Override
    public void onChildMoved(DataSnapshot dataSnapshot, String s) {
    }

    @Override
    public void onCancelled(DatabaseError databaseError) {
    }

    @Override public int getItemCount() {
        return items.size();
    }

    public DatabaseReference getRef(int pos) {
        return items.get(pos).getRef();
    }

    public Student getItem(int pos) {
        return items.get(pos).getValue(Student.class);
    }

    // Replaces '.' with ',' to store in the database
    public static String EncodeString(String string) {
        return string.replace(".", ",");
    }


    // Replace ',' with '.'
    public static String DecodeString(String string) {
        return string.replace(",", ".");
    }

}
