package com.example.supermanoul.swimncoach.adapters;

import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import com.example.supermanoul.swimncoach.Fragments.ShowResultsFragment;
import com.example.supermanoul.swimncoach.POJO.Results;
import com.example.supermanoul.swimncoach.POJO.Student;
import com.example.supermanoul.swimncoach.POJO.TrainingSession;
import com.example.supermanoul.swimncoach.R;
import com.example.supermanoul.swimncoach.activities.Aplicacion;
import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.facebook.FacebookSdk.getApplicationContext;



public class TrainingSessionSelectedAdapter extends RecyclerView.Adapter<TrainingSessionSelectedAdapter.ViewHolder> implements ChildEventListener{
    private LayoutInflater inflater;
    private List <Student> studentsInTS = new ArrayList<Student>();
    private ArrayList<String> keys = new ArrayList<String>();
    private ArrayList<DataSnapshot> items = new ArrayList<DataSnapshot>();
    private DatabaseReference studentsReference;
    private DatabaseReference statsReference;
    private DatabaseReference statsReferenceALL;
    private DatabaseReference trainingSessionsReference;
    private CountDownTimer timer;
    private boolean timerInit = false;
    private boolean timerEnded = false;
    private TrainingSession trainingSession;
    private Context ctx;
    private Map<String, ArrayList> results = new HashMap<String, ArrayList>();
    private Map<String, ArrayList> timesInMS = new HashMap<String, ArrayList>();
    private Map<String, ArrayList> paceInMS = new HashMap<String, ArrayList>();
    private Map<String, Long> totalTimeInMs = new HashMap<String, Long>();
    private Map<String, Double> averagePaceInMs = new HashMap<String, Double>();
    final SimpleDateFormat formatter = new SimpleDateFormat("mm:ss.S");
    final SimpleDateFormat formatter2 = new SimpleDateFormat("mm:ss");
    private FragmentManager fm;


    private Button endSessionButton;
    
    private static final int PACEPARAMETER = 100;

    private int nTimersFinished;



    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder
    public static class ViewHolder extends RecyclerView.ViewHolder {
        TextView studentName;
        TextView lapTime;
        TextView lastLapTime;
        TextView nLapsDistance;

        public ViewHolder(View itemView) {

            super(itemView);
            studentName = (TextView) itemView.findViewById(R.id.studentTSName);
            lapTime = (TextView) itemView.findViewById(R.id.time_per_lap);
            lastLapTime = (TextView) itemView.findViewById(R.id.last_lap_time);
            nLapsDistance = (TextView) itemView.findViewById(R.id.n_laps_distance);

        }

    }

    // Provide a suitable constructor (depends on the kind of dataset)
    public TrainingSessionSelectedAdapter(Context context, DatabaseReference reference, CountDownTimer timer, TrainingSession trainingSession, Context ctx, Button endSessionButton, FragmentManager fm) {
        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();
        studentsReference = reference;
        inflater = LayoutInflater.from(context);
        studentsReference.addChildEventListener(this);
        this.timer = timer;
        this.trainingSession = trainingSession;
        this.ctx = ctx;
        this.endSessionButton = endSessionButton;
        this.fm = fm;
        statsReference = ((Aplicacion) getApplicationContext()).getStudentsStatsReference();
        statsReferenceALL = ((Aplicacion) getApplicationContext()).getStudentsStatsReference();
        trainingSessionsReference = ((Aplicacion) getApplicationContext()).getTrainingSessionsReference();
        nTimersFinished = 0;

    }

    // Create new views (invoked by the layout manager)
    @Override
    public TrainingSessionSelectedAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = inflater.inflate(R.layout.student_ts_row, parent, false);
        TrainingSessionSelectedAdapter.ViewHolder holder = new TrainingSessionSelectedAdapter.ViewHolder(view);
        return holder;
    }

    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(final TrainingSessionSelectedAdapter.ViewHolder holder, int position) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element
        final Student student = getItem(position);

        holder.studentName.setText(student.getName());

        final String individualStudentID = student.getId();

        final String lastLapString = getApplicationContext().getResources().getString(R.string.last_lap);

        final String nLapsDistanceString = getApplicationContext().getResources().getString(R.string.n_laps_distance);

        holder.lastLapTime.setText(String.format(lastLapString, "00:00.0"));

        holder.nLapsDistance.setText(String.format(nLapsDistanceString, 0, 0));

        final int durationInMs = trainingSession.getDuration()*60*1000;

        final int poolLength = trainingSession.getPoolSize();

        final ArrayList<String> individualResults= new ArrayList<String>();

        final ArrayList<Double> individualResultsInMs = new ArrayList<Double>();

        final ArrayList<Double> individualPaceInMs = new ArrayList<Double>();

        final Long[] individualTotalTimeInMs = new Long[1];

        final Double[] individualAveragePaceInMs = new Double[1];

        final int [] nLapsDistance = {-1};
        final int [] individualTotalDistance = {-poolLength};

        final CountDownTimer cdt = new CountDownTimer(durationInMs, 100) {

            Date d = new Date(0);
            Date paceDate = new Date(0);
            String timePerLap = "00:00.0";
            String paceString = "00:00.0";
            long localTime = 0;
            long totalTime = 0;
            long pace = 0;
            long totalPace = 0;

            @Override
            public void onTick(long millisUntilFinished) {

                localTime = durationInMs-millisUntilFinished;
                d = new Date(localTime);
                timePerLap = formatter.format(d);
                holder.lapTime.setText(timePerLap);

            }

            @Override
            public void onFinish() {
                //holder.lapTime.setText("00:00.0");
                individualTotalDistance[0]+=poolLength;
                nLapsDistance[0]+=1;
                holder.nLapsDistance.setText(String.format(nLapsDistanceString, nLapsDistance[0], individualTotalDistance[0]));
                holder.lastLapTime.setText(String.format(lastLapString, timePerLap));
                individualResults.add(timePerLap);
                individualResultsInMs.add((double) localTime/1000);

                totalTime+=localTime;
                individualTotalTimeInMs[0] = totalTime;

                pace = (localTime * PACEPARAMETER)/poolLength;
                totalPace+=pace;
                individualAveragePaceInMs[0] = (double) totalPace;

                paceDate = new Date(pace);
                paceString = formatter.format(paceDate);
                individualPaceInMs.add((double) pace/1000);


            }
        };

        holder.itemView.setOnClickListener(new View.OnClickListener() {

            int ms = 0;

            @Override
            public void onClick(View v) {

                if(!timerInit){
                    timer.start();
                    ms = (int) System.currentTimeMillis();
                    timerInit = true;
                    endSessionButton.setVisibility(View.VISIBLE);
                    endSessionButton.setEnabled(true);
                }

                int ms2 = (int) System.currentTimeMillis();
                int elapsedTime = (int) ms2 - ms;
                ms = ms2;

                cdt.onFinish();
                cdt.cancel();

                if(!timerEnded){
                    cdt.start();
                }

                else {
                    nTimersFinished++;
                    individualResults.remove(0);
                    individualResultsInMs.remove(0);
                    individualPaceInMs.remove(0);
                    results.put(individualStudentID, individualResults);
                    timesInMS.put(individualStudentID, individualResultsInMs);
                    totalTimeInMs.put(individualStudentID, individualTotalTimeInMs[0]);
                    paceInMS.put(individualStudentID, individualPaceInMs);
                    averagePaceInMs.put(individualStudentID, (individualAveragePaceInMs[0]/individualPaceInMs.size())/1000);


                    if(nTimersFinished == items.size()){
                        //saveResults();
                        showSaveDialog();
                    }

                }

            }

        });


        endSessionButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AlertDialog.Builder builder = new AlertDialog.Builder(ctx);
                builder.setTitle(R.string.ad_EndSession);
                builder.setMessage(R.string.ad_askEndAndSaveResults);
                builder.setCancelable(true);


                builder.setPositiveButton(
                        R.string.ad_yes,
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                saveResults();
                                dialog.cancel();
                            }
                        });

                builder.setNegativeButton(
                        R.string.ad_no,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.cancel();
                            }
                        });

                AlertDialog alert = builder.create();
                alert.show();
            }
        });
    }

    @Override
    public void onChildAdded(DataSnapshot dataSnapshot, String s) {
        items.add(dataSnapshot);
        keys.add(dataSnapshot.getKey());
        studentsInTS.add(dataSnapshot.getValue(Student.class));
        notifyItemInserted(getItemCount()-1);
    }

    @Override
    public void onChildChanged(DataSnapshot dataSnapshot, String s) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            items.set(index, dataSnapshot);
            studentsInTS.set(index, dataSnapshot.getValue(Student.class));
            notifyItemChanged(index, dataSnapshot.getValue(Student.class));
        }
    }

    @Override
    public void onChildRemoved(DataSnapshot dataSnapshot) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index != -1) {
            keys.remove(index);
            items.remove(index);
            studentsInTS.remove(index);
            notifyItemRemoved(index);
        }
    }

    @Override
    public void onChildMoved(DataSnapshot dataSnapshot, String s) {
    }

    @Override
    public void onCancelled(DatabaseError databaseError) {
    }

    @Override public int getItemCount() {
        return items.size();
    }

    public DatabaseReference getRef(int pos) {
        return items.get(pos).getRef();
    }

    public Student getItem(int pos) {
        return items.get(pos).getValue(Student.class);
    }

    public List<Student> getStudentsInTS(){return studentsInTS;}

    // Replaces '.' with ',' to store in the database
    public static String EncodeString(String string) {
        return string.replace(".", ",");
    }


    // Replace ',' with '.'
    public static String DecodeString(String string) {
        return string.replace(",", ".");
    }

    public void setTimerEnded(boolean b){
        this.timerEnded = b;
    }

    public void saveResults(){

        int total_distance = 0;
        long total_time;
        String total_time_string;
        double avgPaceInMs;
        double avgSpeed;

        String studentID = "";
        String studentName = "";
        ArrayList studentTimeInString = new ArrayList<String>();
        ArrayList studentTimeInMs = new ArrayList<Double>();
        ArrayList studentPaceInMs = new ArrayList<Double>();
        DatabaseReference studentStatsReference, studentStatsReferenceALL, tsReference, tsReferenceALL;

        trainingSession.setFinished(true);

        tsReference = trainingSessionsReference.child(trainingSession.getGroup()).child(trainingSession.getId());
        tsReference.setValue(trainingSession);

        tsReferenceALL = trainingSessionsReference.child("ALL").child(trainingSession.getId());
        tsReferenceALL.setValue(trainingSession);

        Results resultsStudent;
        
        for (int i=0; i < results.size(); i++){
            studentID = keys.get(i);
            studentName = items.get(i).getValue(Student.class).getName();
            studentTimeInString = results.get(studentID);
            studentTimeInMs = timesInMS.get(studentID);
            studentPaceInMs = paceInMS.get(studentID);
            total_time = totalTimeInMs.get(studentID);
            total_time_string = formatter2.format(new Date(total_time));
            total_distance = studentTimeInString.size()*trainingSession.getPoolSize();
            avgPaceInMs = averagePaceInMs.get(studentID);
            avgSpeed = (double) total_distance*3.6 / (double) (total_time/1000);

            resultsStudent = new Results(studentName, studentTimeInString, studentTimeInMs, studentPaceInMs, (double) total_time/1000, total_distance, total_time_string, avgPaceInMs, avgSpeed);

            studentStatsReference = statsReference.child(studentID).child("groups").child(trainingSession.getGroup()).child(trainingSession.getId());
            studentStatsReference.setValue(trainingSession);
            studentStatsReference.child("results").setValue(resultsStudent);

            studentStatsReferenceALL = statsReferenceALL.child(studentID).child("groups").child("ALL").child(trainingSession.getId());
            studentStatsReferenceALL.setValue(trainingSession);
            studentStatsReferenceALL.child("results").setValue(resultsStudent);

            tsReference.child("results").child(studentID).setValue(resultsStudent);

            tsReferenceALL.child("results").child(studentID).setValue(resultsStudent);

        }

        //Send tsID to next fragment
        Bundle b = new Bundle();
        b.putString("training_session_id", trainingSession.getId());
        b.putString("groupName", trainingSession.getGroup());

        ShowResultsFragment fragment = new ShowResultsFragment();
        fragment.setArguments(b);

        FragmentTransaction ft = fm.beginTransaction();
        ft.replace(R.id.fragment_container, fragment);
        ft.commit();

    }

    private void showSaveDialog(){
        AlertDialog.Builder builder = new AlertDialog.Builder(ctx);
        builder.setTitle(R.string.ad_sessionFinished);
        builder.setMessage(R.string.ad_saveResultsToDatabase);
        builder.setCancelable(true);


        builder.setPositiveButton(
                R.string.ad_yes,
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        saveResults();
                        dialog.cancel();
                    }
                });

        builder.setNegativeButton(
                R.string.ad_no,
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.cancel();
                    }
                });

        AlertDialog alert = builder.create();
        alert.show();
    }



}
